"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var Job_1 = __importDefault(require("./Job"));
var yaml = require('js-yaml');
var Configuration = (function () {
    function Configuration() {
        this.jobs = [];
        this.dockers = [];
    }
    Configuration.prototype.docker = function (image, config) {
        if (config === void 0) { config = {}; }
        var dockerConfig = __assign({ image: image }, config);
        if (this.jobs.length > 0) {
            this.lastJob().dockers.push(dockerConfig);
        }
        else {
            this.dockers.push(dockerConfig);
        }
        return this;
    };
    Configuration.prototype.define = function (name) {
        this.jobs.push(new Job_1.default(name));
        return this;
    };
    Configuration.prototype.lastJob = function () {
        return this.jobs.slice(-1)[0];
    };
    Configuration.prototype.branches = function () {
        var branches = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            branches[_i] = arguments[_i];
        }
        this.lastJob().branches = branches;
        return this;
    };
    Configuration.prototype.requires = function () {
        var jobNames = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            jobNames[_i] = arguments[_i];
        }
        this.lastJob().requires = jobNames;
        return this;
    };
    Configuration.prototype.tasks = function () {
        var literal = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            literal[_i] = arguments[_i];
        }
        this.lastJob().tasksLiteral = literal[0];
        return this;
    };
    Configuration.prototype.usePackage = function (pm) {
        this.lastJob().package = pm;
        return this;
    };
    Configuration.prototype.toConfig = function () {
        var _this = this;
        return {
            version: 2,
            jobs: this.jobs.reduce(function (jobs, job) {
                var _a;
                return (__assign({}, jobs, (_a = {}, _a[job.name] = __assign({ docker: _this.dockers.concat(job.dockers) }, job.toConfig()), _a)));
            }, {}),
            workflows: {
                version: 2,
                master_jobs: {
                    jobs: this.jobs.map(function (j) { return j.toWorkflowConfig(); }),
                },
            },
        };
    };
    Configuration.prototype.dump = function () {
        return yaml.safeDump(this.toConfig(), {
            noRefs: true,
            noCompatMode: true,
        });
    };
    return Configuration;
}());
exports.default = Configuration;
//# sourceMappingURL=Configuration.js.map